from django.db import models
from polymorphic_tree.models import PolymorphicMPTTModel
from ..managers.polymorphic_active_manager import PolymorphicActiveManager


class PolymorphicActiveMixin(PolymorphicMPTTModel):
    is_active = models.BooleanField(default=True, verbose_name='Включено')
    active_objects = PolymorphicActiveManager()

    class Meta:
        abstract = True



from django.http import FileResponse, Http404
from rest_framework import viewsets
from rest_framework.decorators import action
from rest_framework.generics import get_object_or_404


class SecretFileViewMixin(viewsets.GenericViewSet):

    @action(methods=['get'], detail=False, url_path='(?P<share_hash>[^/.]+)')
    def get_file_path(self, request, share_hash, *args, **kwargs):

        instance = get_object_or_404(self.get_queryset(), share_hash=share_hash)

        try:
            return FileResponse(open(instance.file.path, 'rb'))

        except IOError:
            raise Http404



from django.urls import reverse
from rest_framework import serializers


class SecretFileSerializerMixin(serializers.Serializer):
    file_url = serializers.SerializerMethodField(read_only=True)

    def get_file_url(self, obj):
        return self.context['request'].build_absolute_uri(
            reverse(f'{self.view_basename}-get-file-path', kwargs={'share_hash': obj.share_hash}))



import random
import string


def get_random_string(size=8, chars=string.ascii_uppercase + string.digits):
    """
    Получает случайную строку указанного размера и с указанными символами.
    """
    return ''.join(random.SystemRandom().choice(chars) for _ in range(size))


class GenerateHash:

    def __init__(self, hash_length=32):
        self.hash_length = hash_length

    def __call__(self):
        return get_random_string(self.hash_length, chars=string.ascii_letters + string.digits)

    def deconstruct(self):
        return ('garpix_utils.string.GenerateHash', [self.hash_length], {})



from .active_manager import ActiveManager
from .available_manager import AvailableManager
from .polymorphic_active_manager import PolymorphicActiveManager
from .polymorphic_available_manager import PolymorphicAvailableManager
from .current_site_manager import GCurrentSiteManager
from .polymorphic_current_site_manager import GPolymorphicCurrentSiteManager
from .active_on_site_manager import ActiveOnSiteManager



from django.conf import settings
from django.contrib.sites.managers import CurrentSiteManager
from polymorphic_tree.managers import PolymorphicMPTTModelManager


class GPolymorphicCurrentSiteManager(CurrentSiteManager, PolymorphicMPTTModelManager):
    use_in_migrations = False

    def get_queryset(self):
        qs = self.queryset_class(self.model, using=self._db, hints=self._hints)
        if self.model._meta.proxy:
            qs = qs.instance_of(self.model)
        return qs.filter(**{self._get_field_name() + '__id': getattr(settings, 'SITE_ID', 1)})



from django.contrib import admin
from django.contrib.sites.models import Site


class GarpixSiteConfigurationAdmin(admin.ModelAdmin):
    exclude = ('site',)

    def has_add_permission(self, request):
        return False

    def has_delete_permission(self, request, obj=None):
        return False

    def get_queryset(self, request):
        ModelClass = self.model
        ids = ModelClass.objects.all().values_list('site', flat=True)
        sites = Site.objects.exclude(id__in=ids)
        for site in sites:
            ModelClass.objects.create(site=site)
        return super().get_queryset(request)



from .delete_mixin import DeleteMixin
from .active_mixin import ActiveMixin
from ..managers.available_manager import AvailableManager


class AvailableMixin(ActiveMixin, DeleteMixin):
    available_objects = AvailableManager()

    class Meta:
        abstract = True
